@@ -1,6 +1,19 @@
 # vim: set fileencoding=utf-8 :
 #
 # (C) 2007,2009 Guido Guenther <agx@sigxcpu.org>
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 Simple class wrappers for the various external commands needed by
 git-buildpackage and friends
@@ -129,33 +142,30 @@ class PristineTar(Command):
 
 
 class UnpackTarArchive(Command):
-    """Wrap tar to Unpack a gzipped tar archive"""
-    def __init__(self, archive, dir, filters=[]):
+    """Wrap tar to unpack a compressed tar archive"""
+    def __init__(self, archive, dir, filters=[], compression=None):
         self.archive = archive
         self.dir = dir
         exclude = [("--exclude=%s" % filter) for filter in filters]
 
-        if archive.lower().endswith(".bz2"):
-            decompress = "--bzip2"
-        else:
-            decompress = "--gzip"
+        if not compression:
+            compression = '-a'
 
-        Command.__init__(self, 'tar', exclude + ['-C', dir, decompress, '-xf', archive ])
+        Command.__init__(self, 'tar', exclude + ['-C', dir, compression, '-xf', archive ])
         self.run_error = 'Couldn\'t unpack "%s"' % self.archive
 
 
-class RepackTarArchive(Command):
-    """Wrap tar to Repack a gzipped tar archive"""
-    def __init__(self, archive, dir, dest):
+class PackTarArchive(Command):
+    """Wrap tar to pack a compressed tar archive"""
+    def __init__(self, archive, dir, dest, filters=[], compression=None):
         self.archive = archive
         self.dir = dir
+        exclude = [("--exclude=%s" % filter) for filter in filters]
 
-        if archive.lower().endswith(".bz2"):
-            compress = "--bzip2"
-        else:
-            compress = "--gzip"
+        if not compression:
+            compression = '-a'
 
-        Command.__init__(self, 'tar', ['-C', dir, compress, '-cf', archive, dest])
+        Command.__init__(self, 'tar', exclude + ['-C', dir, compression, '-cf', archive, dest])
         self.run_error = 'Couldn\'t repack "%s"' % self.archive
 
 
@@ -200,6 +210,16 @@ class DpkgSourceExtract(Command):
         Command.__call__(self, [dsc, output_dir])
 
 
+class UnpackZipArchive(Command):
+    """Wrap zip to Unpack a zip file"""
+    def __init__(self, archive, dir):
+        self.archive = archive
+        self.dir = dir
+
+        Command.__init__(self, 'unzip', [ "-q", archive, '-d', dir ])
+        self.run_error = 'Couldn\'t unpack "%s"' % self.archive
+
+
 class GitCommand(Command):
     "Mother/Father of all git commands"
     def __init__(self, cmd, args=[], **kwargs):
@@ -290,7 +310,9 @@ def copy_from(orig_dir, filters=[]):
     """
     copy a source tree over via tar
     @param orig_dir: where to copy from
-    @param exclude: tar exclude pattern
+    @type orig_dir: string
+    @param filters: tar exclude pattern
+    @type filters: list of strings
     @return: list of copied files
     @rtype: list
     """
