@@ -1,6 +1,19 @@
 # vim: set fileencoding=utf-8 :
 #
 # (C) 2006,2007 Guido Guenther <agx@sigxcpu.org>
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """provides some debian source package related helpers"""
 
 import commands
@@ -83,9 +96,150 @@ class DpkgCompareVersions(gbpc.Command):
         return 0
 
 
+class UpstreamSource(object):
+    """
+    Upstream source. Can be either an unpacked dir, a tarball or another type
+    or archive
+
+    @cvar is_dir: are the upstream sources an unpacked dir
+    @type is_dir: boolean
+    @cvar _orig: are the upstream sources already suitable as an upstream
+                 tarball
+    @type _irog: boolen
+    @cvar _path: path to the upstream sources
+    @type _path: string
+    @cvar _unpacked: path to the unpacked source tree
+    @type _unpacked: string
+    """
+    def __init__(self, name, unpacked=None):
+        self.is_dir = False
+        self._orig = False
+        self._path = name
+        self.unpacked = unpacked
+
+        self.is_dir = [False, True][os.path.isdir(name)]
+        self._check_orig()
+        if self.is_dir:
+            self.unpacked = self.path
+
+    def _check_orig(self):
+        """Check if archive can be used as orig tarball"""
+        if self.is_dir:
+            self._orig = False
+            return
+
+        parts = self._path.split('.')
+        try:
+            if parts[-2] == 'tar':
+                if (parts[-1] in compressor_opts or
+                    parts[-1] in compressor_aliases):
+                        self._orig = True
+        except IndexError:
+            self._orig = False
+
+    @property
+    def is_orig(self):
+        return self._orig
+
+    @property
+    def path(self):
+        return self._path.rstrip('/')
+
+    def unpack(self, dir, filters=[]):
+        """
+        Unpack packed upstream sources into a given directory
+        and determine the toplevel of the source tree.
+        """
+        if self.is_dir:
+            raise GbpError, "Cannot unpack directory %s" % self.path
+
+        if not filters:
+            filters = []
+
+        if type(filters) != type([]):
+            raise GbpError, "Filters must be a list"
+
+        self._unpack_archive(dir, filters)
+        self.unpacked = self._unpacked_toplevel(dir)
+
+    def _unpack_archive(self, dir, filters):
+        """
+        Unpack packed upstream sources into a given directory.
+        """
+        ext = os.path.splitext(self.path)[1]
+        if ext in [ ".zip", ".xpi" ]:
+            self._unpack_zip(dir)
+        else:
+            self._unpack_tar(dir, filters)
+
+    def _unpack_zip(self, dir):
+        try:
+            gbpc.UnpackZipArchive(self.path, dir)()
+        except gbpc.CommandExecFailed:
+            raise GbpError, "Unpacking of %s failed" % self.path
+
+    def _unpacked_toplevel(self, dir):
+        """unpacked archives can contain a leading directory not"""
+        unpacked = glob.glob('%s/*' % dir)
+        unpacked.extend(glob.glob("%s/.*" % dir)) # include hidden files and folders
+        # Check that dir contains nothing but a single folder:
+        if len(unpacked) == 1 and os.path.isdir(unpacked[0]):
+            return unpacked[0]
+        else:
+            return dir
+
+    def _unpack_tar(self, dir, filters):
+        """
+        unpack a .orig.tar.gz to tmpdir, leave the cleanup to the caller in case of
+        an error
+        """
+        try:
+            unpackArchive = gbpc.UnpackTarArchive(self.path, dir, filters)
+            unpackArchive()
+        except gbpc.CommandExecFailed:
+            # unpackArchive already printed an error message
+            raise GbpError
+
+    def pack(self, newarchive, filters=[]):
+        """
+        recreate a new archive from the current one
+
+        @param newarchive: the name of the new archive
+        @type newarchive: string
+        @param filters: tar filters to apply
+        @type filters: array of strings
+        @return: the new upstream source
+        @rtype: UpstreamSource
+        """
+        if not self.unpacked:
+            raise GbpError, "Need an unpacked source tree to pack"
+
+        if not filters:
+            filters = []
+
+        if type(filters) != type([]):
+            raise GbpError, "Filters must be a list"
+
+        try:
+            unpacked = self.unpacked.rstrip('/')
+            repackArchive = gbpc.PackTarArchive(newarchive,
+                                os.path.dirname(unpacked),
+                                os.path.basename(unpacked),
+                                filters)
+            repackArchive()
+        except gbpc.CommandExecFailed:
+            # repackArchive already printed an error
+            raise GbpError
+        return UpstreamSource(newarchive)
+
+    @staticmethod
+    def known_compressions():
+        return [ args[1][-1] for args in compressor_opts.items() ]
+
+
 class DscFile(object):
     """Keeps all needed data read from a dscfile"""
-    compressions = r"(gz|bz2)"
+    compressions = r"(%s)" % '|'.join(UpstreamSource.known_compressions())
     pkg_re = re.compile(r'Source:\s+(?P<pkg>.+)\s*')
     version_re = re.compile(r'Version:\s((?P<epoch>\d+)\:)?(?P<version>[%s]+)\s*$' % debian_version_chars)
     tar_re = re.compile(r'^\s\w+\s\d+\s+(?P<tar>[^_]+_[^_]+(\.orig)?\.tar\.%s)' % compressions)
@@ -257,6 +411,7 @@ def orig_file(cp, compression):
 def is_native(cp):
     """
     Is this a debian native package
+
     >>> is_native(dict(Version="1"))
     True
     >>> is_native(dict(Version="1-1"))
@@ -275,6 +430,7 @@ def is_valid_upstreamversion(version):
 def get_compression(orig_file):
     """
     Given an orig file return the compression used
+
     >>> get_compression("abc.tar.gz")
     'gzip'
     >>> get_compression("abc.tar.bz2")
@@ -336,71 +492,67 @@ def symlink_orig(cp, compression, orig_dir, output_dir, force=False):
     return True
 
 
-def do_uscan():
-    """invoke uscan to fetch a new upstream version"""
-    p = subprocess.Popen(['uscan', '--symlink', '--destdir=..', '--dehs'], stdout=subprocess.PIPE)
-    out = p.communicate()[0].split('\n')
+def parse_uscan(out):
+    """
+    Parse the uscan output return (True, tarball) if a new version was
+    downloaded and could be located. If the tarball can't be located it returns
+    (True, None). Returns (False, None) if the current version is up to date.
+
+    >>> parse_uscan("<status>up to date</status>")
+    (False, None)
+    >>> parse_uscan("<target>virt-viewer_0.4.0.orig.tar.gz</target>")
+    (True, '../virt-viewer_0.4.0.orig.tar.gz')
+
+    @param out: uscan output
+    @type out: string
+    @return: status and tarball name
+    @rtype: tuple
+    """
+    source = None
     if "<status>up to date</status>" in out:
-        # nothing to do.
         return (False, None)
     else:
-        for row in out:
-            if row.startswith('<messages>'):
-                tarball = "../%s" % re.match(".*symlinked ([^\s]*) to it.*", row).group(1)
+        # Check if uscan downloaded something
+        for row in out.split("\n"):
+            # uscan >= 2.10.70 has a target element:
+            m = re.match(r"<target>(.*)</target>", row)
+            if m:
+                source = '../%s' % m.group(1)
                 break
+            elif row.startswith('<messages>'):
+                m = re.match(r".*symlinked ([^\s]+) to it", row)
+                if m:
+                    source = "../%s" % m.group(1)
+                    break
+                m = re.match(r"Successfully downloaded updated package ([^<]+)", row)
+                if m:
+                    source = "../%s" % m.group(1)
+                    break
+        # try to determine the already downloaded sources name
         else:
             d = {}
-            for row in out:
+            for row in out.split("\n"):
                 for n in ('package', 'upstream-version', 'upstream-url'):
                     m = re.match("<%s>(.*)</%s>" % (n,n), row)
                     if m:
                         d[n] = m.group(1)
-                    else:
-                        continue
             d["ext"] = os.path.splitext(d['upstream-url'])[1]
-            tarball = "../%(package)s_%(upstream-version)s.orig.tar%(ext)s" % d
-
-            if not os.path.exists(tarball):
-                return (True, None)
-        return (True, tarball)
+            # We want the name of the orig tarball if possible
+            source = "../%(package)s_%(upstream-version)s.orig.tar%(ext)s" % d
+            if not os.path.exists(source):
+                # Fall back to the sources name otherwise
+                source = "../%s" % d['upstream-url'].rsplit('/',1)[1]
+                print source
+                if not os.path.exists(source):
+                    source = None
+        return (True, source)
 
 
-def unpack_orig(archive, tmpdir, filters):
-    """
-    unpack a .orig.tar.gz to tmpdir, leave the cleanup to the caller in case of
-    an error
-    """
-    try:
-        unpackArchive = gbpc.UnpackTarArchive(archive, tmpdir, filters)
-        unpackArchive()
-    except gbpc.CommandExecFailed:
-        print >>sys.stderr, "Unpacking of %s failed" % archive
-        raise GbpError
-    return unpackArchive.dir
-
-
-def repack_orig(archive, tmpdir, dest):
-    """
-    recreate a new .orig.tar.gz from tmpdir (useful when using filter option)
-    """
-    try:
-        repackArchive = gbpc.RepackTarArchive(archive, tmpdir, dest)
-        repackArchive()
-    except gbpc.CommandExecFailed:
-        print >>sys.stderr, "Failed to create %s" % archive
-        raise GbpError
-    return repackArchive.dir
-
-
-def tar_toplevel(dir):
-    """tar archives can contain a leading directory not"""
-    unpacked = glob.glob('%s/*' % dir)
-    unpacked.extend(glob.glob("%s/.*" % dir)) # include hidden files and folders
-    # Check that dir contains nothing but a single folder:
-    if len(unpacked) == 1 and os.path.isdir(unpacked[0]):
-        return unpacked[0]
-    else:
-        return dir
+def do_uscan():
+    """invoke uscan to fetch a new upstream version"""
+    p = subprocess.Popen(['uscan', '--symlink', '--destdir=..', '--dehs'], stdout=subprocess.PIPE)
+    out = p.communicate()[0]
+    return parse_uscan(out)
 
 
 def get_arch():
@@ -413,9 +565,9 @@ def guess_upstream_version(archive, extra_regex=r''):
     """
     guess the package name and version from the filename of an upstgream
     archive. Returns a tuple with package name and version, or None.
-    @archive: filename to guess to version for
-    @extra_regex: additional regex to apply, needs a 'package' and a
-    'version' group
+    @param archive: filename to guess to version for
+    @param extra_regex: additional regex to apply, needs a 'package' and a
+                        'version' group
 
     >>> guess_upstream_version('foo-bar_0.2.orig.tar.gz')
     ('foo-bar', '0.2')
@@ -434,7 +586,6 @@ def guess_upstream_version(archive, extra_regex=r''):
     ('kvm', '87+dfsg')
     >>> guess_upstream_version('foo-Bar_0.2.orig.tar.gz')
     >>> guess_upstream_version('foo-Bar-a.b.tar.gz')
-
     """
     version_chars = r'[a-zA-Z\d\.\~\-\:\+]'
     extensions = r'\.tar\.(gz|bz2)'
